
import os
import subprocess
import sys
import yaml

root_path = os.path.dirname(os.path.realpath(__file__)) + "/../.."
script_path = root_path + "/script"

sys.path.insert(1, script_path)
from rbc_utils import replace_in_file


### RQC example preprocessing function
def preprocessing_example_rqc(sec_level):
    sec_level = str(sec_level)

    src_path = "./test/example/rqc"
    build_path = "./build/test/example/rqc" + sec_level

    subprocess.run("mkdir -p " + build_path, shell=True)
    subprocess.run("cp -r " + src_path + "/*" + " " + build_path, shell=True)

    replace_in_file(build_path + "/CMakeLists.txt",   "rqc", "rqc_" + sec_level);
    replace_in_file(build_path + "/CMakeLists.txt",   "CONFIG_LIB_DIR", root_path + "/bin");

    replace_in_file(build_path + "/main.c",           "rqc_", "rqc_" + sec_level + "_");
    replace_in_file(build_path + "/main.c",           "RQC_", "RQC_" + sec_level + "_");



### ROLLOI example preprocessing function
def preprocessing_example_rolloI(sec_level):
    sec_level = str(sec_level)

    src_path = "./test/example/rolloI"
    build_path = "./build/test/example/rolloI" + sec_level

    subprocess.run("mkdir -p " + build_path, shell=True)
    subprocess.run("cp -r " + src_path + "/*" + " " + build_path, shell=True)

    replace_in_file(build_path + "/CMakeLists.txt",   "rolloI", "rolloI_" + sec_level);
    replace_in_file(build_path + "/CMakeLists.txt",   "CONFIG_LIB_DIR", root_path + "/bin");

    replace_in_file(build_path + "/main.c",           "rolloI_", "rolloI_" + sec_level + "_");
    replace_in_file(build_path + "/main.c",           "ROLLOI_", "ROLLOI_" + sec_level + "_");



### ROLLOII example preprocessing function
def preprocessing_example_rolloII(sec_level):
    sec_level = str(sec_level)

    src_path = "./test/example/rolloII"
    build_path = "./build/test/example/rolloII" + sec_level

    subprocess.run("mkdir -p " + build_path, shell=True)
    subprocess.run("cp -r " + src_path + "/*" + " " + build_path, shell=True)

    replace_in_file(build_path + "/CMakeLists.txt",   "rolloII", "rolloII_" + sec_level);
    replace_in_file(build_path + "/CMakeLists.txt",   "CONFIG_LIB_DIR", root_path + "/bin");

    replace_in_file(build_path + "/main.c",           "rolloII_", "rolloII_" + sec_level + "_");
    replace_in_file(build_path + "/main.c",           "ROLLOII_", "ROLLOII_" + sec_level + "_");





### Parsing configuration file
config_file = sys.argv[1] if len(sys.argv) > 1 else root_path + "/config.yml"
with open(config_file, 'r') as yaml_file: 
    config = yaml.safe_load(yaml_file)



### RQC example preprocessing
for scheme in config['rbc_build_parameters']:
    if(scheme['name'] == "rqc"):
        preprocessing_example_rqc(scheme['sec_lvl'])

subprocess.run("mkdir -p ./build/test/example/rqc-all", shell=True)
subprocess.run("cp -r ./test/example/rqc-all/* ./build/test/example/rqc-all", shell=True)
replace_in_file("./build/test/example/rqc-all/CMakeLists.txt",   "CONFIG_LIB_DIR", root_path + "/bin");



### ROLLOI example preprocessing
for scheme in config['rbc_build_parameters']:
    if(scheme['name'] == "rolloI"):
        preprocessing_example_rolloI(scheme['sec_lvl'])

subprocess.run("mkdir -p ./build/test/example/rolloI-all", shell=True)
subprocess.run("cp -r ./test/example/rolloI-all/* ./build/test/example/rolloI-all", shell=True)
replace_in_file("./build/test/example/rolloI-all/CMakeLists.txt",   "CONFIG_LIB_DIR", root_path + "/bin");



### ROLLOII example preprocessing
for scheme in config['rbc_build_parameters']:
    if(scheme['name'] == "rolloII"):
        preprocessing_example_rolloII(scheme['sec_lvl'])

subprocess.run("mkdir -p ./build/test/example/rolloII-all", shell=True)
subprocess.run("cp -r ./test/example/rolloII-all/* ./build/test/example/rolloII-all", shell=True)
replace_in_file("./build/test/example/rolloII-all/CMakeLists.txt", "CONFIG_LIB_DIR", root_path + "/bin");

