from math import ceil

def preprocessing_rbc_elt_add(target, m):
    if(target == "ref"):  return preprocessing_c64_rbc_elt_add(m)
    if(target == "avx" and m <= 64):  return preprocessing_avx_rbc_elt_add_m64()
    if(target == "avx" and m <= 128): return preprocessing_avx_rbc_elt_add_m128()
    if(target == "avx" and m <= 192): return preprocessing_avx_rbc_elt_add_m192()



# C64 implementations

def preprocessing_c64_rbc_elt_add(m):
    rbc_elt_uint64 = ceil(m / 64)

    result = ""

    for i in range(0, rbc_elt_uint64):
        result += "  o[" + str(i) + "] = e1[" + str(i) + "] ^ e2[" + str(i) + "];\n"

    return result.rstrip()



# AVX implementations

def preprocessing_avx_rbc_elt_add_m64():
    result = ""

    result += "  o[0] = e1[0] ^ e2[0];\n"

    return result.rstrip()



def preprocessing_avx_rbc_elt_add_m128():
    result = ""

    result += "  __m128i tmp1 = _mm_load_si128((__m128i*) e1);\n"
    result += "  __m128i tmp2 = _mm_load_si128((__m128i*) e2);\n"
    result += "  tmp1 = _mm_xor_si128(tmp1, tmp2);\n"
    result += "  _mm_store_si128((__m128i*) o, tmp1);\n"

    return result.rstrip()



def preprocessing_avx_rbc_elt_add_m192():
    result = ""

    result += "  __m128i tmp1 = _mm_load_si128((__m128i*) e1);\n"
    result += "  __m128i tmp2 = _mm_load_si128((__m128i*) e2);\n"
    result += "  tmp1 = _mm_xor_si128(tmp1, tmp2);\n"
    result += "  _mm_store_si128((__m128i*) o, tmp1);\n"

    result += "\n"
    result += "  tmp1 = _mm_load_si128((__m128i*) (e1 + 2));\n"
    result += "  tmp2 = _mm_load_si128((__m128i*) (e2 + 2));\n"
    result += "  tmp1 = _mm_xor_si128(tmp1, tmp2);\n"
    result += "  _mm_store_si128((__m128i*) (o + 2), tmp1);\n"

    return result.rstrip()



def preprocessing_avx2_rbc_elt_add_m192():
    result = ""

    result += "  __m256i tmp1 = _mm256_loadu_si256((__m256i*) e1);\n"
    result += "  __m256i tmp2 = _mm256_loadu_si256((__m256i*) e2);\n"
    result += "  tmp1 = _mm256_xor_si256(tmp1, tmp2);\n"
    result += "  _mm256_storeu_si256((__m256i*) o, tmp1);\n"

    return result.rstrip()

