from math import ceil

def preprocessing_rbc_elt_reduce(target, m):
    if(m == 61):   return preprocessing_c64_rbc_elt_reduce_m61()
    if(m == 67):   return preprocessing_c64_rbc_elt_reduce_m67()
    if(m == 79):   return preprocessing_c64_rbc_elt_reduce_m79()
    if(m == 83):   return preprocessing_c64_rbc_elt_reduce_m83()
    if(m == 97):   return preprocessing_c64_rbc_elt_reduce_m97()
    if(m == 127):  return preprocessing_c64_rbc_elt_reduce_m127()
    if(m == 151):  return preprocessing_c64_rbc_elt_reduce_m151()
    if(m == 181):  return preprocessing_c64_rbc_elt_reduce_m181()



# C64 implementations

def preprocessing_c64_rbc_elt_reduce_m61():
    result = ""

    result += "  uint64_t tmp = (e[1] << 3) ^ (e[0] >> 61) ^ (e[1] >> 53) ^ (e[1] >> 56);\n"
    result += "  o[0] = e[0] ^ tmp ^ (tmp << 1) ^ (tmp << 2) ^ (tmp << 5);\n"
    result += "  o[0] &= 0x1FFFFFFFFFFFFFFF;\n"

    return result.rstrip()



def preprocessing_c64_rbc_elt_reduce_m67():
    result = ""

    result += "  uint64_t tmp = (e[1] >> 62) ^ (e[2] << 2);\n"
    result += "  o[1] = e[1] ^ tmp ^ (tmp >> 3) ^ (tmp >> 4) ^ (tmp >> 5);\n"

    result += "\n"
    result += "  tmp = (o[1] >> 3) ^ (e[2] << 61);\n"
    result += "  o[0] = e[0] ^ tmp ^ (tmp << 1) ^ (tmp << 2) ^ (tmp << 5);\n"

    result += "\n"
    result += "  o[1] &= 0x0000000000000007;\n"

    return result.rstrip()



def preprocessing_c64_rbc_elt_reduce_m79():
    result = ""

    result += "  uint64_t tmp = e[2] >> 6;\n"
    result += "  o[1] = e[1] ^ tmp ^ (tmp >> 9);\n"

    result += "\n"
    result += "  tmp = (o[1] >> 15) ^ (e[2] << 49);\n"
    result += "  o[0] = e[0] ^ tmp ^ (tmp << 9);\n"

    result += "\n"
    result += "  o[1] &= 0x0000000000007FFF;\n"

    return result.rstrip()



def preprocessing_c64_rbc_elt_reduce_m83():
    result = ""

    result += "  uint64_t tmp = e[2] >> 12;\n"
    result += "  o[1] = e[1] ^ tmp ^ (tmp >> 3) ^ (tmp >> 5) ^ (tmp >> 7);\n"

    result += "\n"
    result += "  tmp = (o[1] >> 19) ^ (e[2] << 45);\n"
    result += "  o[0] = e[0] ^ tmp ^ (tmp << 2) ^ (tmp << 4) ^ (tmp << 7);\n"

    result += "\n"
    result += "  o[1] &= 0x000000000007FFFF;\n"

    return result.rstrip()



def preprocessing_c64_rbc_elt_reduce_m97():
    result = ""

    result += "\n"
    result += "  uint64_t tmp = (e[2] >> 27) ^ (e[3] << 37);\n"
    result += "  o[1] = e[1] ^ tmp ^ (tmp >> 6);\n"

    result += "\n"
    result += "  tmp = (o[1] >> 33) ^ (e[2] << 31);\n"
    result += "  o[0] = e[0] ^ tmp ^ (tmp << 6);\n"

    result += "\n"
    result += "  o[1] &= 0x00000001FFFFFFFF;\n"

    return result.rstrip()



def preprocessing_c64_rbc_elt_reduce_m127():
    result = ""

    result += "  uint64_t tmp = (e[2] >> 62) ^ (e[3] << 2);\n"
    result += "  o[1] = e[1] ^ tmp ^ (tmp >> 1);\n"

    result += "\n"
    result += "  tmp = (o[1] >> 63) ^ (e[2] << 1);\n"
    result += "  o[0] = e[0] ^ tmp ^ (tmp << 1);\n"

    result += "\n"
    result += "  o[1] &= 0x7FFFFFFFFFFFFFFF;\n"

    return result.rstrip()



def preprocessing_c64_rbc_elt_reduce_m151():
    result = ""

    result += "\n"
    result += "  uint64_t tmp = e[4] >> 20;\n"
    result += "  o[2] = e[2] ^ tmp ^ (tmp >> 3);\n"

    result += "\n"
    result += "  tmp = e[3] >> 20;\n"
    result += "  o[1] = e[1] ^ tmp ^ (tmp >> 3);\n"

    result += "  tmp = e[4] << 41;\n"
    result += "  o[1] ^= tmp ^ (tmp << 3);\n"

    result += "\n"
    result += "  tmp = (o[2] >> 23) ^ (e[3] << 41);\n"
    result += "  o[0] = e[0] ^ tmp ^ (tmp << 3);\n"

    result += "\n"
    result += "  o[2] &= 0x00000000007FFFFF;\n"

    return result.rstrip()



def preprocessing_c64_rbc_elt_reduce_m181():
    result = ""

    result += "  uint64_t tmp = e[5] << 11;\n"
    result += "  uint64_t tmp2 = e[4] >> 46;\n"

    result += "\n"
    result += "  o[2] = e[2]\n"
    result += "    ^ tmp ^ (tmp << 1) ^ (tmp << 6) ^ (tmp << 7)\n"
    result += "    ^ tmp2 ^ (tmp2 >> 1) ^ (tmp2 >> 6) ^ (tmp2 >> 7);\n"

    result += "\n"
    result += "  tmp = e[4] << 11;\n"
    result += "  tmp2 = e[3] >> 46;\n"

    result += "\n"
    result += "  o[1] = e[1]\n"
    result += "    ^ tmp ^ (tmp << 1) ^ (tmp << 6) ^ (tmp << 7)\n"
    result += "    ^ tmp2 ^ (tmp2 >> 1) ^ (tmp2 >> 6) ^ (tmp2 >> 7);\n"

    result += "\n"
    result += "  tmp = (e[3] << 11) ^ (o[2] >> 53);\n"
    result += "  o[0] = e[0] ^ tmp ^ (tmp << 1) ^ (tmp << 6) ^ (tmp << 7);\n"

    result += "\n"
    result += "  o[2] &= 0x001FFFFFFFFFFFFF;\n"

    return result.rstrip()

