
import os
from math import ceil, floor


def preprocessing_rbc_elt_set_zero(field_uint64):
    result = ""
    for i in range(0, field_uint64):
        result += "  o[" + str(i) + "] = 0;\n"

    return result.rstrip()



def preprocessing_rbc_elt_set_one(field_uint64):
    result = "  o[0] = 1;\n"
    for i in range(1, field_uint64):
        result += "  o[" + str(i) + "] = 0;\n"

    return result.rstrip()



def preprocessing_rbc_elt_set(field_uint64, last_block_dummy):
    result = ""
    for i in range(0, field_uint64 - 1):
        result += "  o[" + str(i) + "] = e[" + str(i) + "];\n"

    if last_block_dummy == 0:
        result += "  o[" + str(field_uint64 - 1) + "] = e[" + str(field_uint64 - 1) + "];\n"
    else:
        result += "  o[" + str(field_uint64 - 1) + "] = 0;\n"

    return result.rstrip()



def preprocessing_rbc_elt_set_mask1(field_uint64, last_block_dummy):
    result = ""
    for i in range(0, field_uint64 - 1):
        result += "  o[" + str(i) + "] = mask * e1[" + str(i) + "] + (1 - mask) * e2[" + str(i) + "];\n"

    if last_block_dummy == 0:
        result += "  o[" + str(field_uint64 - 1) + "] = mask * e1[" + str(field_uint64 - 1) + "] + (1 - mask) * e2[" + str(field_uint64 - 1) + "];\n"
    else:
        result += "  o[" + str(field_uint64 - 1) + "] = 0;\n"

    return result.rstrip()



def preprocessing_rbc_elt_set_mask2(field_uint64, last_block_dummy):
    result = ""
    for i in range(0, field_uint64 - 1):
        result += "  o1[" + str(i) + "] = mask * e[" + str(i) + "] + (1 - mask) * o1[" + str(i) + "];\n"
        result += "  o2[" + str(i) + "] = mask * e[" + str(i) + "] + (1 - mask) * o2[" + str(i) + "];\n"
        result += "\n"

    if last_block_dummy == 0:
        result += "  o1[" + str(field_uint64 - 1) + "] = mask * e[" + str(field_uint64 - 1) + "] + (1 - mask) * o1[" + str(field_uint64 - 1) + "];\n"
        result += "  o2[" + str(field_uint64 - 1) + "] = mask * e[" + str(field_uint64 - 1) + "] + (1 - mask) * o2[" + str(field_uint64 - 1) + "];\n"
    else:
        result += "  o1[" + str(field_uint64 - 1) + "] = 0;\n"
        result += "  o2[" + str(field_uint64 - 1) + "] = 0;\n"

    return result.rstrip()



def preprocessing_rbc_elt_set_from_uint64(field_uint64, last_block_dummy):
    result = ""
    for i in range(0, field_uint64 - 1):
        result += "  o[" + str(i) + "] = e[" + str(i) + "];\n"

    if last_block_dummy == 0:
        result += "  o[" + str(field_uint64 - 1) + "] = e[" + str(field_uint64 - 1) + "];\n"
    else:
        result += "  o[" + str(field_uint64 - 1) + "] = 0;\n"

    return result.rstrip()



def preprocessing_rbc_elt_is_zero(field_uint64, last_block_dummy):
    start = field_uint64 - 1
    if last_block_dummy == 1: start -= 1

    condition = "e[" + str(start) + "] == 0"
    for i in range(start - 1, -1, -1):
        condition += " && e[" + str(i) + "] == 0"

    return "  return " + condition + ";"



def preprocessing_rbc_elt_is_equal_to(field_uint64, last_block_dummy):
    stop = field_uint64
    if last_block_dummy == 1: stop -= 1

    condition = "e1[0] == e2[0]"
    for i in range(1, stop):
        condition += " && e1[" + str(i) + "] == e2[" + str(i) + "]"

    return "  return " + condition + ";"



def preprocessing_rbc_elt_is_greater_than(field_uint64, last_block_dummy):
    start = field_uint64 - 1
    if last_block_dummy == 1: start -= 1

    result = ""
    result += "  if(e1[" + str(start) + "] > e2[" + str(start) + "]) {\n"
    result += "    return 1;\n"
    result += "  }\n\n"
    result += "  if(e1[" + str(start) + "] < e2[" + str(start) + "]) {\n"
    result += "    return 0;\n"
    result += "  }\n\n"

    for i in range(start - 1, -1, -1):
        condition = ""
        for j in range(start, i, -1):
            condition += "e1[" + str(j) + "] == e2[" + str(j) + "] && "

        condition += "e1[" + str(i) + "] > e2[" + str(i) + "]"
        result += "  if(" + condition + ") {\n    return 1;\n  }\n\n"

    return result + "  return 0;"



def preprocessing_rbc_elt_get_degree(field_uint64, last_block_dummy):
    result = "  int64_t index = 0;\n"
    result += "\n"

    start = field_uint64 - 1
    if last_block_dummy == 1: start -= 1

    for i in range(start, -1, -1):
        result += "  if(e[" + str(i) + "] != 0) {\n" 
        result += "    __asm__ volatile(\"bsr %1,%0;\" : \"=r\"(index) : \"r\"(e[" + str(i) + "]));\n"
        result += "    return index + " + str(64 * i) + ";\n"
        result += "  }\n\n"

    return result + "  return -1;"



def preprocessing_rbc_elt_inv(field_m):
    # Implementation working for 0 <= m < 192 only

    # Euclidean algorithm to find o such that e * o + g * RBC_ELT_MODULUS = 1

    result = ""
    if(field_m >= 0):    result += "  rbc_elt u, v, g, tmp;\n"
    if(field_m >= 0):    result += "  uint32_t u_degree, v_degree;\n"
    if(field_m >= 0):    result += "  uint64_t carry;\n"
    if(field_m >= 0):    result += "  int32_t j;\n"

    result += "\n"
    if(field_m >= 0):    result += "  rbc_elt_set(u, e);\n"
    if(field_m >= 0):    result += "  rbc_elt_set(v, RBC_ELT_MODULUS);\n"

    result += "\n"
    if(field_m >= 0):    result += "  rbc_elt_set_one(o);\n"
    if(field_m >= 0):    result += "  rbc_elt_set_zero(g);\n"

    result += "\n"
    if(field_m >= 0):    result += "  while(u[0] != 1) {\n"
    if(field_m >= 0):    result += "    u_degree = rbc_elt_get_degree(u);\n"
    if(field_m >= 0):    result += "    v_degree = rbc_elt_get_degree(v);\n"
    if(field_m >= 0):    result += "    j = u_degree - v_degree;\n"

    result += "\n"
    if(field_m >= 0):    result += "    if(j < 0) {\n"
    if(field_m >= 0):    result += "      rbc_elt_set(tmp, u);\n"
    if(field_m >= 0):    result += "      rbc_elt_set(u, v);\n"
    if(field_m >= 0):    result += "      rbc_elt_set(v, tmp);\n"

    result += "\n"
    if(field_m >= 0):    result += "      rbc_elt_set(tmp, o);\n"
    if(field_m >= 0):    result += "      rbc_elt_set(o, g);\n"
    if(field_m >= 0):    result += "      rbc_elt_set(g, tmp);\n"

    result += "\n"
    if(field_m >= 0):    result += "      j -= j;\n";
    if(field_m >= 0):    result += "    }\n"

    result += "\n"
    if(field_m >= 0):    result += "    if(j == 0) {\n"
    if(field_m >= 0):    result += "      rbc_elt_add(u, u, v);\n"
    if(field_m >= 0):    result += "      rbc_elt_add(o, o, g);\n"
    if(field_m >= 0):    result += "    }\n"

    result += "\n"
    if(field_m >= 0):    result += "    // 0 < j <= 64\n"
    if(field_m >= 0):    result += "    else if (j <= 64) {\n"
    if(field_m >= 0):    result += "      carry = v[0] >> (64 - j);\n"
    if(field_m >= 0):    result += "      u[0] ^= (v[0] << j);\n"
    if(field_m >= 0):    result += "      u[1] ^= (v[1] << j) ^ carry;\n"
    if(field_m >= 128):  result += "      carry = v[1] >> (64 - j);\n"
    if(field_m >= 128):  result += "      u[2] ^= (v[2] << j) ^ carry;\n"

    result += "\n"
    if(field_m >= 0):    result += "      carry = g[0] >> (64 - j);\n"
    if(field_m >= 0):    result += "      o[0] ^= g[0] << j;\n"
    if(field_m >= 0):    result += "      o[1] ^= (g[1] << j) ^ carry;\n"
    if(field_m >= 128):  result += "      carry = g[1] >> (64 - j);\n"
    if(field_m >= 128):  result += "      o[2] ^= (g[2] << j) ^ carry;\n"
    if(field_m >= 0):    result += "    }\n"

    result += "\n"
    if(field_m < 128):
                          result += "    // 64 < j < M\n"
                          result += "    else {\n"
                          result += "      u[1] ^= v[0] << (j - 64);\n"
                          result += "      o[1] ^= g[0] << (j - 64);\n"
                          result += "    }\n"
    else:
                          result += "    // 64 < j <= 128\n"
                          result += "    else if(j > 64 && j <= 128) {\n"
                          result += "      u[1] ^= v[0] << (j - 64);\n"
                          result += "      u[2] ^= v[0] >> (j - 128);\n"

                          result += "\n"
                          result += "      o[1] ^= g[0] << (j - 64);\n"
                          result += "      o[2] ^= g[0] >> (j - 128);\n"
                          result += "    }\n"

                          result += "\n"
                          result += "    // 128 < j <= M\n"
                          result += "    else {\n"
                          result += "      u[2] ^= v[0] << (j - 128);\n"
                          result += "      o[2] ^= g[0] << (j - 128);\n"
                          result += "    }\n"

    if(field_m >= 0):     result += "  }"

    return result



def preprocessing_rbc_elt_print(field_uint64, last_block_dummy):
    stop = field_uint64
    if last_block_dummy == 1: stop -= 1

    result = "  printf(\"[ \");\n  printf(\"%lu"
    for i in range(1, stop):
        result += " %lu"
    result += "\""

    for i in range(0, stop):
        result += ", e[" + str(i) +"]"

    return result + ");\n  printf(\" ]\");"





def preprocessing_rbc_elt_field_init():
    result = ""

    result += "  static uint32_t init_field = 0;\n"
    result += "  uint8_t bit = 0;\n"
    result += "  uint64_t mask = 0;\n"

    result += "\n"
    result += "  if(init_field == 0) {\n"
    result += "    memset(RBC_SQR_LOOKUP_TABLE, 0, 8*256);\n"
    result += "    for(size_t i = 0 ; i < 256 ; ++i) {\n"
    result += "      for(size_t j = 0 ; j < 8 ; ++j) {\n"
    result += "        mask = 1 << j;\n"
    result += "        bit = (mask & i) >> j;\n"
    result += "        RBC_SQR_LOOKUP_TABLE[i] ^= bit << (2 * j);\n"
    result += "      }\n"
    result += "    }\n"

    result += "\n"
    result += "    init_field++;\n"
    result += "  }\n"

    return result.rstrip()

