
import os
import subprocess
import sys
import yaml
from preprocessing_src_cmake import *
from preprocessing_src_core import *
from preprocessing_src_codes import *
from preprocessing_src_schemes import *

root_path = os.path.dirname(os.path.realpath(__file__)) + "/../.."


### Parsing configuration file
config_file = sys.argv[1] if len(sys.argv) > 1 else root_path + "/config.yml"
with open(config_file, 'r') as yaml_file: 
    config = yaml.safe_load(yaml_file)


### CMakeList preprocessing
preprocessing_src_cmake(config)


### rbc.h include preprocessing
preprocessing_src_rbc_include(config)


### External lib preprocessing
subprocess.run("cp -r ./lib ./build", shell=True)


### Core preprocessing
for scheme in config['rbc_build_parameters']:
    print("    Preprocessing core" + str(scheme['m']))
    preprocessing_src_core(scheme['m'], config['rbc_core_layer_implem'])


# Codes preprocessing
for scheme in config['rbc_build_parameters']:
    if(scheme['name'] == "rqc"):
        print("    Preprocessing gabidulin" + str(scheme['m']))
        preprocessing_src_gabidulin(scheme['m'])

for scheme in config['rbc_build_parameters']:
    if(scheme['name'] == "rolloI" or scheme['name'] == "rolloII" or scheme['name'] == "rolloIII"):
        print("    Preprocessing lrpc" + str(scheme['m']))
        preprocessing_src_lrpc(scheme['m'])


# Schemes preprocessing
for scheme in config['rbc_build_parameters']:
    if(scheme['name'] == "rqc"):
        print("    Preprocessing rqc_" + str(scheme['sec_lvl']))
        preprocessing_src_rqc(scheme['sec_lvl'], scheme['m'])

for scheme in config['rbc_build_parameters']:
    if(scheme['name'] == "rolloI"):
        print("    Preprocessing rolloI_" + str(scheme['sec_lvl']))
        preprocessing_src_rolloI(scheme['sec_lvl'], scheme['m'])

for scheme in config['rbc_build_parameters']:
    if(scheme['name'] == "rolloII"):
        print("    Preprocessing rolloII_" + str(scheme['sec_lvl']))
        preprocessing_src_rolloII(scheme['sec_lvl'], scheme['m'])

