
import os
import subprocess
import sys
import yaml

root_path = os.path.dirname(os.path.realpath(__file__)) + "/../.."
preprocessing_path = root_path + "/script/preprocessing"
script_path = root_path + "/script"

sys.path.insert(1, script_path)
from rbc_utils import *


### Cmake set dir preprocessing
def preprocessing_cmake_set_dir(config):
    result = ""

    for scheme in config['rbc_build_parameters']:
        result += "set(CORE_" + str(scheme['m']) + "_DIR              ${CMAKE_CURRENT_SOURCE_DIR}/src/core/${CORE_LAYER_IMPLEM}/rbc-" + str(scheme['m']) + ")\n"
    
    result += "\n"
    for scheme in config['rbc_build_parameters']:
        if(scheme['name'] == "rqc"):
            result += "set(GABIDULIN_" + str(scheme['m']) + "_DIR         ${CMAKE_CURRENT_SOURCE_DIR}/src/codes/gabidulin-" + str(scheme['m']) + ")\n"
    
    result += "\n"
    for scheme in config['rbc_build_parameters']:
        if(scheme['name'] == "rolloI" or scheme['name'] == "rolloII"):
            result += "set(LRPC_" + str(scheme['m']) + "_DIR              ${CMAKE_CURRENT_SOURCE_DIR}/src/codes/lrpc-" + str(scheme['m']) + ")\n"

    result += "\n"
    for scheme in config['rbc_build_parameters']:
        result += "set(" + str(scheme['name']).upper() + str(scheme['sec_lvl']) + "_DIR           ${CMAKE_CURRENT_SOURCE_DIR}/src/schemes/" + str(scheme['name']) + str(scheme['sec_lvl']) + ")\n"

    return result.rstrip()



### Cmake add subdirectory preprocessing
def preprocessing_cmake_add_subdirectory(config):
    result = ""

    for scheme in config['rbc_build_parameters']:
        result += "add_subdirectory(src/core/${CORE_LAYER_IMPLEM}/rbc-" + str(scheme['m']) + ")\n"

    result += "\n"
    for scheme in config['rbc_build_parameters']:
        if(scheme['name'] == "rqc"):
            result += "add_subdirectory(src/codes/gabidulin-" + str(scheme['m']) + ")\n"

    result += "\n"
    for scheme in config['rbc_build_parameters']:
        if(scheme['name'] == "rolloI" or scheme['name'] == "rolloII"):
            result += "add_subdirectory(src/codes/lrpc-" + str(scheme['m']) + ")\n"

    result += "\n"
    for scheme in config['rbc_build_parameters']:
        result += "add_subdirectory(src/schemes/" + str(scheme['name']) + str(scheme['sec_lvl']) + ")\n"

    return remove_duplicates(result).rstrip()



### Cmake add library preprocessing
def preprocessing_cmake_add_library(config):
    result = ""

    for scheme in config['rbc_build_parameters']:
        result += "  $<TARGET_OBJECTS:core_" + str(scheme['m']) + ">\n"

    result += "\n"
    for scheme in config['rbc_build_parameters']:
        if(scheme['name'] == "rqc"):
            result += "  $<TARGET_OBJECTS:gabidulin_" + str(scheme['m']) + ">\n"

    result += "\n"
    for scheme in config['rbc_build_parameters']:
        if(scheme['name'] == "rolloI" or scheme['name'] == "rolloII"):
            result += "  $<TARGET_OBJECTS:lrpc_" + str(scheme['m']) + ">\n"

    result += "\n"
    for scheme in config['rbc_build_parameters']:
        result += "  $<TARGET_OBJECTS:" + scheme['name'] + "_" + str(scheme['sec_lvl']) + ">\n"

    return result.rstrip()



### Cmake install file preprocessing
def preprocessing_cmake_install_files(config):
    result = ""

    for scheme in config['rbc_build_parameters']:
        result += "install(FILES ${RBC_LIB_SRC}/core/${CORE_LAYER_IMPLEM}/rbc-" + str(scheme['m']) + "/rbc_" + str(scheme['m']) + ".h          DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
        result += "install(FILES ${RBC_LIB_SRC}/core/${CORE_LAYER_IMPLEM}/rbc-" + str(scheme['m']) + "/rbc_" + str(scheme['m']) + "_elt.h      DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
        result += "install(FILES ${RBC_LIB_SRC}/core/${CORE_LAYER_IMPLEM}/rbc-" + str(scheme['m']) + "/rbc_" + str(scheme['m']) + "_vec.h      DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
        result += "install(FILES ${RBC_LIB_SRC}/core/${CORE_LAYER_IMPLEM}/rbc-" + str(scheme['m']) + "/rbc_" + str(scheme['m']) + "_poly.h     DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
        result += "install(FILES ${RBC_LIB_SRC}/core/${CORE_LAYER_IMPLEM}/rbc-" + str(scheme['m']) + "/rbc_" + str(scheme['m']) + "_qre.h      DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
        result += "install(FILES ${RBC_LIB_SRC}/core/${CORE_LAYER_IMPLEM}/rbc-" + str(scheme['m']) + "/rbc_" + str(scheme['m']) + "_vspace.h   DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
        result += "\n"

    for scheme in config['rbc_build_parameters']:
        if(scheme['name'] == "rqc"):
            result += "install(FILES ${RBC_LIB_SRC}/codes/gabidulin-" + str(scheme['m']) + "/rbc_" + str(scheme['m']) + "_qpoly.h                  DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
            result += "install(FILES ${RBC_LIB_SRC}/codes/gabidulin-" + str(scheme['m']) + "/rbc_" + str(scheme['m']) + "_gabidulin.h              DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
            result += "\n"

    for scheme in config['rbc_build_parameters']:
        if(scheme['name'] == "rolloI" or scheme['name'] == "rolloII"):
            result += "install(FILES ${RBC_LIB_SRC}/codes/lrpc-" + str(scheme['m']) + "/rbc_" + str(scheme['m']) + "_lrpc.h                        DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"

    result += "\n"
    for scheme in config['rbc_build_parameters']:
        if(scheme['name'] == "rqc"):
            result += "install(FILES ${RBC_LIB_SRC}/schemes/rqc" + str(scheme['sec_lvl']) + "/rqc_" + str(scheme['sec_lvl']) + "_parameters.h           DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
            result += "install(FILES ${RBC_LIB_SRC}/schemes/rqc" + str(scheme['sec_lvl']) + "/rqc_" + str(scheme['sec_lvl']) + "_parsing.h              DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
            result += "install(FILES ${RBC_LIB_SRC}/schemes/rqc" + str(scheme['sec_lvl']) + "/rqc_" + str(scheme['sec_lvl']) + "_pke.h                  DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
            result += "install(FILES ${RBC_LIB_SRC}/schemes/rqc" + str(scheme['sec_lvl']) + "/rqc_" + str(scheme['sec_lvl']) + "_kem.h                  DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
            result += "\n"

    for scheme in config['rbc_build_parameters']:
        if(scheme['name'] == "rolloI"):
            result += "install(FILES ${RBC_LIB_SRC}/schemes/rolloI" + str(scheme['sec_lvl']) + "/rolloI_" + str(scheme['sec_lvl']) + "_parameters.h DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
            result += "install(FILES ${RBC_LIB_SRC}/schemes/rolloI" + str(scheme['sec_lvl']) + "/rolloI_" + str(scheme['sec_lvl']) + "_parsing.h    DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
            result += "install(FILES ${RBC_LIB_SRC}/schemes/rolloI" + str(scheme['sec_lvl']) + "/rolloI_" + str(scheme['sec_lvl']) + "_types.h      DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
            result += "install(FILES ${RBC_LIB_SRC}/schemes/rolloI" + str(scheme['sec_lvl']) + "/rolloI_" + str(scheme['sec_lvl']) + "_kem.h        DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
            result += "\n"

        if(scheme['name'] == "rolloII"):
            result += "install(FILES ${RBC_LIB_SRC}/schemes/rolloII" + str(scheme['sec_lvl']) + "/rolloII_" + str(scheme['sec_lvl']) + "_parameters.h DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
            result += "install(FILES ${RBC_LIB_SRC}/schemes/rolloII" + str(scheme['sec_lvl']) + "/rolloII_" + str(scheme['sec_lvl']) + "_parsing.h    DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
            result += "install(FILES ${RBC_LIB_SRC}/schemes/rolloII" + str(scheme['sec_lvl']) + "/rolloII_" + str(scheme['sec_lvl']) + "_types.h      DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
            result += "install(FILES ${RBC_LIB_SRC}/schemes/rolloII" + str(scheme['sec_lvl']) + "/rolloII_" + str(scheme['sec_lvl']) + "_kem.h        DESTINATION ${RBC_LIB_INSTALL_PATH_INC})\n"
            result += "\n"

    return result.rstrip()



### Global rbc.h preprocessing
def preprocessing_src_rbc_include(config):
    # Preprocessing for src
    result =""

    result += "#ifndef RBC_LIB_H\n"
    result += "#define RBC_LIB_H\n"
    result += "\n"

    for scheme in config['rbc_build_parameters']:
        result += "#include \"rbc_" + str(scheme['m']) + ".h\"\n"
        result += "#include \"rbc_" + str(scheme['m']) + "_elt.h\"\n"
        result += "#include \"rbc_" + str(scheme['m']) + "_vec.h\"\n"
        result += "#include \"rbc_" + str(scheme['m']) + "_poly.h\"\n"
        result += "#include \"rbc_" + str(scheme['m']) + "_qre.h\"\n"
        result += "#include \"rbc_" + str(scheme['m']) + "_vspace.h\"\n"
        result += "\n"

    for scheme in config['rbc_build_parameters']:
        if(scheme['name'] == "rqc"):
            result += "#include \"rbc_" + str(scheme['m']) + "_qpoly.h\"\n"
            result += "#include \"rbc_" + str(scheme['m']) + "_gabidulin.h\"\n"
            result += "\n"

    for scheme in config['rbc_build_parameters']:
        if(scheme['name'] == "rolloI" or scheme['name'] == "rolloII"):
            result += "#include \"rbc_" + str(scheme['m']) + "_lrpc.h\"\n"
            result += "\n"

    for scheme in config['rbc_build_parameters']:
        if(scheme['name'] == "rqc"):
            result += "#include \"rqc_" + str(scheme['sec_lvl']) + "_parameters.h\"\n"
            result += "#include \"rqc_" + str(scheme['sec_lvl']) + "_parsing.h\"\n"
            result += "#include \"rqc_" + str(scheme['sec_lvl']) + "_pke.h\"\n"
            result += "#include \"rqc_" + str(scheme['sec_lvl']) + "_kem.h\"\n"
            result += "\n"

    for scheme in config['rbc_build_parameters']:
        if(scheme['name'] == "rolloI"):
            result += "#include \"rolloI_" + str(scheme['sec_lvl']) + "_parameters.h\"\n"
            result += "#include \"rolloI_" + str(scheme['sec_lvl']) + "_parsing.h\"\n"
            result += "#include \"rolloI_" + str(scheme['sec_lvl']) + "_types.h\"\n"
            result += "#include \"rolloI_" + str(scheme['sec_lvl']) + "_kem.h\"\n"
            result += "\n"

        if(scheme['name'] == "rolloII"):
            result += "#include \"rolloII_" + str(scheme['sec_lvl']) + "_parameters.h\"\n"
            result += "#include \"rolloII_" + str(scheme['sec_lvl']) + "_parsing.h\"\n"
            result += "#include \"rolloII_" + str(scheme['sec_lvl']) + "_types.h\"\n"
            result += "#include \"rolloII_" + str(scheme['sec_lvl']) + "_kem.h\"\n"
            result += "\n"

    result += "#endif\n"

    subprocess.run("mkdir -p " + root_path + "/src/include", shell=True)
    with open(root_path + "/src/include/rbc.h", 'w') as file:
        file.write(result)
        file.close()

    # Preprocessing for build
    subprocess.run("mkdir -p " + root_path + "/build/src/include", shell=True)
    copy_file(root_path + "/src/include/rbc.h", root_path + "/build/src/include/rbc.h")



def preprocessing_src_cmake(config):
    # Preprocessing for src
    copy_file(preprocessing_path + "/CMakeLists.txt", root_path + "/src/CMakeLists.txt")

    replace_in_file(root_path + "/src/CMakeLists.txt", "{TEMPLATING_CMAKE_SET_DIR}", preprocessing_cmake_set_dir(config))
    replace_in_file(root_path + "/src/CMakeLists.txt", "{TEMPLATING_CMAKE_ADD_SUBDIRECTORY}", preprocessing_cmake_add_subdirectory(config))
    replace_in_file(root_path + "/src/CMakeLists.txt", "{TEMPLATING_CMAKE_ADD_LIBRARY}", preprocessing_cmake_add_library(config))
    replace_in_file(root_path + "/src/CMakeLists.txt", "{TEMPLATING_CMAKE_INSTALL_FILES}", preprocessing_cmake_install_files(config))
    replace_in_file(root_path + "/src/CMakeLists.txt", "CONFIG_CORE_LAYER_IMPLEM", config['rbc_core_layer_implem']);
    replace_in_file(root_path + "/src/CMakeLists.txt", "CONFIG_LIB_DIR", root_path + "/bin");

    # Preprocessing for build
    subprocess.run("mkdir -p " + root_path + "/build/src/include", shell=True)
    copy_file(root_path + "/src/CMakeLists.txt", root_path + "/build")




