
import os
import subprocess
import sys
import yaml
from math import ceil
from preprocessing_rbc_elt_utils import *
from preprocessing_rbc_elt_add import *
from preprocessing_rbc_elt_mul import *
from preprocessing_rbc_elt_sqr import *
from preprocessing_rbc_elt_reduce import *


root_path = os.path.dirname(os.path.realpath(__file__)) + "/../.."
core_path = root_path + "/src/core"
script_path = root_path + "/script"
preprocessing_path = script_path + "/preprocessing"

sys.path.insert(1, script_path)
from rbc_utils import *



### Field modulus
def preprocessing_src_core_get_modulus(m):
    field_modulus = ""
    if(m == 61):  field_modulus = "{0x2000000000000027}"                                          # X^61 + X^5 + X^2 + X + 1
    if(m == 67):  field_modulus = "{0x0000000000000027, 0x0000000000000008}"                      # X^67 + X^5 + X^2 + X + 1
    if(m == 79):  field_modulus = "{0x0000000000000201, 0x0000000000008000}"                      # X^79 + X^9 + 1
    if(m == 83):  field_modulus = "{0x0000000000000095, 0x0000000000080000}"                      # X^83 + X^7 + X^4 + X^2 + 1
    if(m == 89):  field_modulus = "{0x0000004000000001, 0x0000000002000000}"                      # X^89 + X^38 + 1
    if(m == 97):  field_modulus = "{0x0000000000000041, 0x0000000200000000}"                      # X^97 + X^6 + 1
    if(m == 101): field_modulus = "{0x00000000000000c3, 0x0000002000000000}"                      # X^101 + X^7 + X^6 + X + 1
    if(m == 107): field_modulus = "{0x0000000000000291, 0x0000080000000000}"                      # X^107 + X^9 + X^7 + X^4 + 1
    if(m == 113): field_modulus = "{0x0000000000000201, 0x0002000000000000}"                      # X^113 + X^9 + 1
    if(m == 127): field_modulus = "{0x0000000000000003, 0x8000000000000000}"                      # X^127 + X + 1
    if(m == 131): field_modulus = "{0x000000000000010d, 0x0000000000000000, 0x0000000000000008}"  # X^131 + X^8 + X^3 + X^2 + 1
    if(m == 137): field_modulus = "{0x0000000000200001, 0x0000000000000000, 0x0000000000000200}"  # X^137 + X^21 + 1
    if(m == 151): field_modulus = "{0x0000000000000009, 0x0000000000000000, 0x0000000000800000}"  # X^151 + X^3 + 1
    if(m == 181): field_modulus = "{0x00000000000000c3, 0x0000000000000000, 0x0020000000000000}"  # X^181 + X^7 + X^6 + X + 1

    return field_modulus



### Core header preprocessing function
def preprocessing_src_core_header(m, implem):
    # Preprocessing to src
    output_file = core_path + "/" + implem + "/rbc-" + str(m) + "/rbc.h"
    copy_file(preprocessing_path + "/core/rbc.h", output_file)

    rbc_elt_uint64 = 0
    rbc_elt_ur_uint64 = 0

    if(implem == "ref"): 
        rbc_elt_uint64 = ceil(m / 64)
        rbc_elt_ur_uint64 = ceil((2 * m - 1) / 64)

    if(implem == "avx"):
        rbc_elt_uint64 = 2 * ceil(m / 128)
        rbc_elt_ur_uint64 = 2 * ceil((2 * m - 1) / 128)

    constants = ""
    if(implem == "ref"): 
        constants += "uint64_t RBC_SQR_LOOKUP_TABLE[256];\n"
        constants += "static const rbc_elt RBC_ELT_MODULUS = {TEMPLATING_RBC_ELT_MODULUS};\n"

    if(implem == "avx"):
        constants += "static const __m128i RBC_ELT_SQR_MASK_128 = {0x0F0F0F0F0F0F0F0F, 0x0F0F0F0F0F0F0F0F};\n"
        constants += "static const __m128i RBC_ELT_SQR_LOOKUP_TABLE_128 = {0x1514111005040100, 0x5554515045444140};\n"
        constants += "static const rbc_elt RBC_ELT_MODULUS = {TEMPLATING_RBC_ELT_MODULUS};"

    replace_in_file(output_file, "{TEMPLATING_RBC_HEADER_CONSTANTS}", constants)

    replace_in_file(output_file, "{TEMPLATING_FIELD_Q}", "2")
    replace_in_file(output_file, "{TEMPLATING_FIELD_M}", str(m))
    replace_in_file(output_file, "{TEMPLATING_ELT_UINT64}", str(rbc_elt_uint64))
    replace_in_file(output_file, "{TEMPLATING_ELT_UR_UINT64}", str(rbc_elt_ur_uint64))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_MODULUS}", preprocessing_src_core_get_modulus(m))

    # Preprocessing to build
    build_path = root_path + "/build/src/core/" + implem + "/rbc-" + str(m)
    copy_file(output_file, build_path + "/rbc_" + str(m) + ".h")
    replace_in_file(build_path + "/rbc_" + str(m) + ".h", "rbc", "rbc_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + ".h", "RBC", "RBC_" + str(m));



### Core elt preprocessing function
def preprocessing_src_core_elt(m, implem):
    # Preprocessing to src
    output_path = core_path + "/" + implem + "/rbc-" + str(m)
    copy_file(preprocessing_path + "/core/rbc_elt.c", output_path + "/rbc_elt.c")
    copy_file(preprocessing_path + "/core/rbc_elt.h", output_path + "/rbc_elt.h")

    rbc_elt_uint64 = 0
    rbc_elt_ur_uint64 = 0

    if(implem == "ref"): 
        rbc_elt_uint64 = ceil(m / 64)
        rbc_elt_ur_uint64 = ceil((2 * m - 1) / 64)

    if(implem == "avx"): 
        rbc_elt_uint64 = 2 * ceil(m / 128)
        rbc_elt_ur_uint64 = 2 * ceil((2 * m - 1) / 128)

    rbc_elt_uint64_last_dummy = 1 if ceil(m / 64) != rbc_elt_uint64 else 0;
    rbc_elt_ur_uint64_last_dummy = 1 if ceil((2 * m - 1) / 64) != rbc_elt_ur_uint64 else 0;

    output_file = output_path + "/rbc_elt.c"

    if(implem == "avx"): 
        replace_in_file(output_file, "{TEMPLATING_RBC_ELT_FIELD_INIT}", "")

    if(implem == "ref"): 
        replace_in_file(output_file, "{TEMPLATING_RBC_ELT_FIELD_INIT}",     preprocessing_rbc_elt_field_init())

    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_SET_ZERO}",           preprocessing_rbc_elt_set_zero(rbc_elt_uint64))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_SET_ONE}",            preprocessing_rbc_elt_set_one(rbc_elt_uint64))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_SET}",                preprocessing_rbc_elt_set(rbc_elt_uint64, rbc_elt_uint64_last_dummy))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_SET_MASK1}",          preprocessing_rbc_elt_set_mask1(rbc_elt_uint64, rbc_elt_uint64_last_dummy))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_SET_MASK2}",          preprocessing_rbc_elt_set_mask2(rbc_elt_uint64, rbc_elt_uint64_last_dummy))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_IS_ZERO}",            preprocessing_rbc_elt_is_zero(rbc_elt_uint64, rbc_elt_uint64_last_dummy))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_IS_EQUAL_TO}",        preprocessing_rbc_elt_is_equal_to(rbc_elt_uint64, rbc_elt_uint64_last_dummy))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_IS_GREATER_THAN}",    preprocessing_rbc_elt_is_greater_than(rbc_elt_uint64, rbc_elt_uint64_last_dummy))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_GET_DEGREE}",         preprocessing_rbc_elt_get_degree(rbc_elt_uint64, rbc_elt_uint64_last_dummy))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_PRINT}",              preprocessing_rbc_elt_print(rbc_elt_uint64, rbc_elt_uint64_last_dummy))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_UR_PRINT}",           preprocessing_rbc_elt_print(rbc_elt_ur_uint64, rbc_elt_ur_uint64_last_dummy))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_SET_FROM_UINT64}",    preprocessing_rbc_elt_set_from_uint64(rbc_elt_uint64, rbc_elt_uint64_last_dummy))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_UR_SET_FROM_UINT64}", preprocessing_rbc_elt_set_from_uint64(rbc_elt_ur_uint64, rbc_elt_ur_uint64_last_dummy))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_UR_SET_ZERO}",        preprocessing_rbc_elt_set_zero(rbc_elt_ur_uint64))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_ADD}",                preprocessing_rbc_elt_add(implem, m))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_REDUCE}",             preprocessing_rbc_elt_reduce(implem, m))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_UR_MUL}",             preprocessing_rbc_elt_mul(implem, m))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_UR_SQR}",             preprocessing_rbc_elt_sqr(implem, m))
    replace_in_file(output_file, "{TEMPLATING_RBC_ELT_INV}",                preprocessing_rbc_elt_inv(m))

    # Preprocessing to build
    build_path = root_path + "/build/src/core/" + implem + "/rbc-" + str(m)
    copy_file(output_path + "/rbc_elt.h", build_path + "/rbc_" + str(m) + "_elt.h")
    copy_file(output_path + "/rbc_elt.c", build_path + "/rbc_" + str(m) + "_elt.c")
    replace_in_file(build_path + "/rbc_" + str(m) + "_elt.h", "rbc", "rbc_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_elt.h", "RBC", "RBC_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_elt.c", "rbc", "rbc_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_elt.c", "RBC", "RBC_" + str(m));



### Core vec preprocessing function
def preprocessing_src_core_vec(m, implem):
    # Preprocessing to src
    output_path = core_path + "/" + implem + "/rbc-" + str(m)
    copy_file(preprocessing_path + "/core/rbc_vec.h", output_path + "/rbc_vec.h")
    copy_file(preprocessing_path + "/core/rbc_vec.c", output_path + "/rbc_vec.c")

    # Preprocessing to build
    build_path = root_path + "/build/src/core/" + implem + "/rbc-" + str(m)
    copy_file(output_path + "/rbc_vec.h", build_path + "/rbc_" + str(m) + "_vec.h")
    copy_file(output_path + "/rbc_vec.c", build_path + "/rbc_" + str(m) + "_vec.c")
    replace_in_file(build_path + "/rbc_" + str(m) + "_vec.h", "rbc", "rbc_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_vec.h", "RBC", "RBC_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_vec.c", "rbc", "rbc_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_vec.c", "RBC", "RBC_" + str(m));



### Core poly preprocessing function
def preprocessing_src_core_poly(m, implem):
    # Preprocessing to src
    output_path = core_path + "/" + implem + "/rbc-" + str(m)
    copy_file(preprocessing_path + "/core/rbc_poly.h", output_path + "/rbc_poly.h")
    copy_file(preprocessing_path + "/core/rbc_poly.c", output_path + "/rbc_poly.c")

    # Preprocessing to build
    build_path = root_path + "/build/src/core/" + implem + "/rbc-" + str(m)
    copy_file(output_path + "/rbc_poly.h", build_path + "/rbc_" + str(m) + "_poly.h")
    copy_file(output_path + "/rbc_poly.c", build_path + "/rbc_" + str(m) + "_poly.c")
    replace_in_file(build_path + "/rbc_" + str(m) + "_poly.h", "rbc", "rbc_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_poly.h", "RBC", "RBC_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_poly.c", "rbc", "rbc_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_poly.c", "RBC", "RBC_" + str(m));



### Core qre preprocessing function
def preprocessing_src_core_qre(m, implem):
    # Preprocessing to src
    output_path = core_path + "/" + implem + "/rbc-" + str(m)
    copy_file(preprocessing_path + "/core/rbc_qre.h", output_path + "/rbc_qre.h")
    copy_file(preprocessing_path + "/core/rbc_qre.c", output_path + "/rbc_qre.c")

    # Preprocessing to build
    build_path = root_path + "/build/src/core/" + implem + "/rbc-" + str(m)
    copy_file(output_path + "/rbc_qre.h", build_path + "/rbc_" + str(m) + "_qre.h")
    copy_file(output_path + "/rbc_qre.c", build_path + "/rbc_" + str(m) + "_qre.c")
    replace_in_file(build_path + "/rbc_" + str(m) + "_qre.h", "rbc", "rbc_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_qre.h", "RBC", "RBC_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_qre.c", "rbc", "rbc_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_qre.c", "RBC", "RBC_" + str(m));



### Core vspace preprocessing function
def preprocessing_src_core_vspace(m, implem):
    # Preprocessing to src
    output_path = core_path + "/" + implem + "/rbc-" + str(m)
    copy_file(preprocessing_path + "/core/rbc_vspace.h", output_path + "/rbc_vspace.h")
    copy_file(preprocessing_path + "/core/rbc_vspace.c", output_path + "/rbc_vspace.c")

    # Preprocessing to build
    build_path = root_path + "/build/src/core/" + implem + "/rbc-" + str(m)
    copy_file(output_path + "/rbc_vspace.h", build_path + "/rbc_" + str(m) + "_vspace.h")
    copy_file(output_path + "/rbc_vspace.c", build_path + "/rbc_" + str(m) + "_vspace.c")
    replace_in_file(build_path + "/rbc_" + str(m) + "_vspace.h", "rbc", "rbc_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_vspace.h", "RBC", "RBC_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_vspace.c", "rbc", "rbc_" + str(m));
    replace_in_file(build_path + "/rbc_" + str(m) + "_vspace.c", "RBC", "RBC_" + str(m));



### Core preprocessing function
def preprocessing_src_core(m, implem):
    subprocess.run("mkdir -p " + core_path + "/" + implem + "/rbc-" + str(m), shell=True)
    subprocess.run("mkdir -p " + root_path + "/build/src/core/" + implem + "/rbc-" + str(m), shell=True)

    copy_file(core_path + "/" + implem + "/CMakeLists.txt", root_path + "/build/src/core/" + implem + "/rbc-" + str(m))
    replace_in_file(root_path + "/build/src/core/" + implem + "/rbc-" + str(m) + "/CMakeLists.txt", "core", "core_" + str(m));
    replace_in_file(root_path + "/build/src/core/" + implem + "/rbc-" + str(m) + "/CMakeLists.txt", "rbc", "rbc_" + str(m));

    preprocessing_src_core_header(m, implem)
    preprocessing_src_core_elt(m, implem)
    preprocessing_src_core_vec(m, implem)
    preprocessing_src_core_poly(m, implem)
    preprocessing_src_core_qre(m, implem)
    preprocessing_src_core_vspace(m, implem)

