
import os
import subprocess
import sys
import yaml

root_path = os.path.dirname(os.path.realpath(__file__)) + "/../.."
script_path = root_path + "/script"

sys.path.insert(1, script_path)
from rbc_utils import replace_in_file


### Test lib preprocessing
def preprocessing_test_lib():
    if(os.path.isdir("./build/lib/mpfq") == False):
        subprocess.run("mkdir -p ./build/lib", shell=True)
        subprocess.run("./script/lib-mpfq-compile", shell=True)

    subprocess.run("mkdir -p ./build/lib/minunit", shell=True)
    subprocess.run("cp -r ./lib/minunit/* ./build/lib/minunit", shell=True)


### Unit test elt preprocessing function
def preprocessing_unit_test_elt(param_m):
    param_m = str(param_m)

    src_path = "./test/unit_test/"
    build_path = "./build/test/unit_test/rbc_" + param_m + "_elt"
    test_file_path = build_path + "/test_rbc_" + param_m + "_elt.c"

    subprocess.run("mkdir -p " + build_path, shell=True)
    subprocess.run("cp -r " + src_path + "rbc_elt/*" + " " + build_path, shell=True)
    subprocess.run("cp -r " + src_path + "utils.h" + " " + build_path, shell=True)
    subprocess.run("mv " + build_path + "/test_rbc_elt.c" + " " + test_file_path, shell=True)

    replace_in_file(build_path + "/CMakeLists.txt",     "rbc_",  "rbc_" + param_m + "_");
    replace_in_file(build_path + "/CMakeLists.txt",     "CONFIG_LIB_DIR", root_path + "/bin");
    replace_in_file(build_path + "/CMakeLists.txt",     "mpfq_", "mpfq_2_" + param_m);

    replace_in_file(build_path + "/utils.h",            "mpfq_", "mpfq_2_" + param_m + "_");
    replace_in_file(build_path + "/utils.h",            "rbc_",  "rbc_" + param_m + "_");
    replace_in_file(build_path + "/utils.h",            "RBC_",  "RBC_" + param_m + "_");

    replace_in_file(test_file_path,                     "mpfq_", "mpfq_2_" + param_m + "_");
    replace_in_file(test_file_path,                     "mpfq.h",  "mpfq_2_" + param_m + ".h");
    replace_in_file(test_file_path,                     "rbc_",  "rbc_" + param_m + "_");
    replace_in_file(test_file_path,                     "RBC_",  "RBC_" + param_m + "_");


### Unit test poly preprocessing function
def preprocessing_unit_test_poly(param_m, param_n):
    param_m = str(param_m)
    param_n = str(param_n)

    src_path = "./test/unit_test/"
    build_path = "./build/test/unit_test/rbc_" + param_m + "_poly_" + param_n
    test_file_path = build_path + "/test_rbc_" + param_m + "_poly_" + param_n + ".c"

    subprocess.run("mkdir -p " + build_path, shell=True)
    subprocess.run("cp -r " + src_path + "rbc_poly/*" + " " + build_path, shell=True)
    subprocess.run("cp -r " + src_path + "utils.h" + " " + build_path, shell=True)
    subprocess.run("mv " + build_path + "/test_rbc_poly.c" + " " + test_file_path, shell=True)

    replace_in_file(build_path + "/CMakeLists.txt",     "rbc_poly",  "rbc_" + param_m + "_poly_" + param_n);
    replace_in_file(build_path + "/CMakeLists.txt",     "CONFIG_LIB_DIR", root_path + "/bin");
    replace_in_file(build_path + "/CMakeLists.txt",     "mpfq_", "mpfq_2_" + param_m);

    replace_in_file(build_path + "/utils.h",            "mpfq_", "mpfq_2_" + param_m + "_");
    replace_in_file(build_path + "/utils.h",            "rbc_",  "rbc_" + param_m + "_");
    replace_in_file(build_path + "/utils.h",            "RBC_",  "RBC_" + param_m + "_");

    replace_in_file(test_file_path,                     "mpfq_", "mpfq_2_" + param_m + "_");
    replace_in_file(test_file_path,                     "mpfq.h",  "mpfq_2_" + param_m + ".h");
    replace_in_file(test_file_path,                     "rbc_",  "rbc_" + param_m + "_");
    replace_in_file(test_file_path,                     "RBC_",  "RBC_" + param_m + "_");
    replace_in_file(test_file_path,                     "DEGREE;",  "DEGREE = " + param_n + ";");


### Unit test vspace preprocessing function
def preprocessing_unit_test_vspace(param_m):
    param_m = str(param_m)

    src_path = "./test/unit_test/"
    build_path = "./build/test/unit_test/rbc_" + param_m + "_vspace"
    test_file_path = build_path + "/test_rbc_" + param_m + "_vspace.c"

    subprocess.run("mkdir -p " + build_path, shell=True)
    subprocess.run("cp -r " + src_path + "rbc_vspace/*" + " " + build_path, shell=True)
    subprocess.run("cp -r " + src_path + "utils.h" + " " + build_path, shell=True)
    subprocess.run("mv " + build_path + "/test_rbc_vspace.c" + " " + test_file_path, shell=True)

    replace_in_file(build_path + "/CMakeLists.txt",     "rbc_",  "rbc_" + param_m + "_");
    replace_in_file(build_path + "/CMakeLists.txt",     "CONFIG_LIB_DIR", root_path + "/bin");
    replace_in_file(build_path + "/CMakeLists.txt",     "mpfq_", "mpfq_2_" + param_m);

    replace_in_file(build_path + "/utils.h",            "mpfq_", "mpfq_2_" + param_m + "_");
    replace_in_file(build_path + "/utils.h",            "rbc_",  "rbc_" + param_m + "_");
    replace_in_file(build_path + "/utils.h",            "RBC_",  "RBC_" + param_m + "_");

    replace_in_file(test_file_path,                     "mpfq_", "mpfq_2_" + param_m + "_");
    replace_in_file(test_file_path,                     "mpfq.h",  "mpfq_2_" + param_m + ".h");
    replace_in_file(test_file_path,                     "rbc_",  "rbc_" + param_m + "_");
    replace_in_file(test_file_path,                     "RBC_",  "RBC_" + param_m + "_");





### Parsing configuration file
config_file = sys.argv[1] if len(sys.argv) > 1 else root_path + "/config.yml"
with open(config_file, 'r') as yaml_file: 
    config = yaml.safe_load(yaml_file)


### Unit test preprocessing
preprocessing_test_lib()

for scheme in config['rbc_build_parameters']:
    preprocessing_unit_test_elt(scheme['m'])
    preprocessing_unit_test_poly(scheme['m'], scheme['n'])
    preprocessing_unit_test_vspace(scheme['m'])

