/*
 * Unit-tests against MPFQ
 */
#include <unistd.h>
#include <syscall.h>
#include <stdlib.h>
#include "minunit.h"
#include "mpfq.h"
#include "rbc.h"
#include "utils.h"



/*
 * Variables
 */
int INSTANCES = 10;
int DEGREE;

mpfq_field field1;
mpfq_poly a1;
mpfq_poly b1;
mpfq_poly c1;

mpfq_poly c1_tmp;
mpfq_poly modulo1;
mpfq_poly sparse_modulo1;

mpfq_poly divisor1;
mpfq_poly q1;
mpfq_poly r1;

rbc_poly a2;
rbc_poly b2;
rbc_poly c2;

rbc_poly modulo2;
rbc_poly_sparse sparse_modulo2;

rbc_poly divisor2;
rbc_poly q2;
rbc_poly r2;

#define DIVISOR_SIZE 10


/*
 * Initialization and destruction
 */
void test_setup() {

  // MPFQ initialization
  mpfq_field_init(field1);
  mp_limb_t base = 2;
  mpfq_field_setopt(field1, MPFQ_IO_TYPE, &base);

  mpfq_poly_init(field1, a1, DEGREE-1);
  mpfq_poly_init(field1, b1, DEGREE-1);
  mpfq_poly_init(field1, c1, DEGREE-1);
  mpfq_poly_set_random(field1, a1, DEGREE-1);
  mpfq_poly_set_random(field1, b1, DEGREE-1);
  mpfq_poly_set_zero(field1, c1, DEGREE-1);

  mpfq_poly_init(field1, c1_tmp, 2*DEGREE-1);
  mpfq_poly_init(field1, modulo1, DEGREE);
  mpfq_poly_set_random(field1, modulo1, DEGREE);
  mpfq_poly_set_zero(field1, c1_tmp, 2*DEGREE-1);

  mpfq_poly_init(field1, sparse_modulo1, DEGREE);
  mpfq_poly_set_zero(field1, sparse_modulo1, DEGREE);

  if(DEGREE == 47) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 5);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 47);
  } else if(DEGREE == 53) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 1);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 2);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 6);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 53);
  } else if(DEGREE == 59) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 2);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 4);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 7);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 59);
  } else if(DEGREE == 67) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 1);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 2);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 5);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 67);
  } else if(DEGREE == 83) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 2);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 4);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 7);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 83);
  } else if(DEGREE == 97) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 6);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 97);
  } else if(DEGREE == 101) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 1);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 6);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 7);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 101);
  } else if(DEGREE == 113) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 9);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 113);
  } else if(DEGREE == 131) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 2);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 3);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 8);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 131);
  } else if(DEGREE == 149) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 7);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 9);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 10);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 149);
  } else if(DEGREE == 151) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 3);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 151);
  } else if(DEGREE == 157) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 2);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 5);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 6);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 157);
  } else if(DEGREE == 179) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 1);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 2);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 4);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 179);
  } else if(DEGREE == 189) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 2);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 5);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 6);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 189);
  } else if(DEGREE == 193) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 15);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 193);
  } else if(DEGREE == 211) {
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 0);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 8);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 10);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 11);
    mpfq_poly_setcoef_ui(field1, sparse_modulo1, 1, 211);
  }

  mpfq_poly_init(field1, divisor1, DIVISOR_SIZE-1);
  mpfq_poly_init(field1, q1, DEGREE-1);
  mpfq_poly_init(field1, r1, DEGREE-1);
  mpfq_poly_set_random(field1, divisor1, DIVISOR_SIZE-1);
  mpfq_poly_set_zero(field1, q1, DEGREE-1);
  mpfq_poly_set_zero(field1, r1, DEGREE-1);

  // RBC initialization
  rbc_field_init();

  rbc_poly_init(&a2, DEGREE-1);
  rbc_poly_init(&b2, DEGREE-1);
  rbc_poly_init(&c2, DEGREE-1);
  rbc_poly_set_mpfq(a2, a1, DEGREE-1);
  rbc_poly_set_mpfq(b2, b1, DEGREE-1);
  rbc_poly_set_mpfq(c2, c1, DEGREE-1);

  rbc_poly_init(&modulo2, DEGREE);
  rbc_poly_set_mpfq(modulo2, modulo1, DEGREE);

  sparse_modulo2 = rbc_qre_get_modulus(DEGREE);

  rbc_poly_init(&divisor2, DIVISOR_SIZE-1);
  rbc_poly_init(&q2, DEGREE-1);
  rbc_poly_init(&r2, DEGREE-1);
  rbc_poly_set_mpfq(divisor2, divisor1, DIVISOR_SIZE-1);
  rbc_poly_set_mpfq(q2, q1, DEGREE-1);
  rbc_poly_set_mpfq(r2, r1, DEGREE-1);

  #if VERBOSE
    printf("\na1:"); mpfq_poly_print_uint64(a1);
    printf("\na2:"); rbc_poly_print(a2);
    printf("\n");
    printf("\nb1:"); mpfq_poly_print_uint64(b1);
    printf("\nb2:"); rbc_poly_print(b2);
    printf("\n");
  #endif
}

void test_teardown() {};

void MU_SUITE_CLEAR() {
  mpfq_poly_clear(field1, a1);
  mpfq_poly_clear(field1, b1);
  mpfq_poly_clear(field1, c1);
  mpfq_poly_clear(field1, c1_tmp);
  mpfq_poly_clear(field1, modulo1);
  mpfq_poly_clear(field1, sparse_modulo1);
  mpfq_poly_clear(field1, divisor1);
  mpfq_poly_clear(field1, q1);
  mpfq_poly_clear(field1, r1);
  mpfq_field_clear(field1);

  rbc_poly_clear(a2);
  rbc_poly_clear(b2);
  rbc_poly_clear(c2);
  rbc_poly_clear(modulo2);
  rbc_poly_sparse_clear(sparse_modulo2);
  rbc_poly_clear(divisor2);
  rbc_poly_clear(q2);
  rbc_poly_clear(r2);
}



/*
 * Test functions
 */
MU_TEST(test_poly_add) {
  mpfq_poly_add(field1, c1, a1, b1);
  rbc_poly_add(c2, a2, b2);

  #if VERBOSE
    printf("\nc1:"); mpfq_poly_print_uint64(c1);
    printf("\nc2:"); rbc_poly_print(c2);
    printf("\n");
  #endif

  mu_assert(rbc_poly_equal_to_mpfq(c1, c2) == 1, "Failure: test_poly_add");
}



MU_TEST(test_poly_mul) {
  mpfq_poly_mul(field1, c1, b1, a1);
  rbc_poly_mul(c2, a2, b2);

  #if VERBOSE
    printf("\nc1:"); mpfq_poly_print_uint64(c1);
    printf("\nc2:"); rbc_poly_print(c2);
    printf("\n");
  #endif

  mu_assert(rbc_poly_equal_to_mpfq(c1, c2) == 1, "Failure: test_poly_mul");
}



MU_TEST(test_poly_mulmod_sparse) {
  mpfq_poly quotient;
  mpfq_poly_init(field1, quotient, DEGREE);
  mpfq_poly_mul(field1, c1_tmp, b1, a1);
  mpfq_poly_divmod(field1, quotient, c1, c1_tmp, sparse_modulo1);
  mpfq_poly_clear(field1, quotient);

  rbc_poly_mulmod_sparse(c2, a2, b2, sparse_modulo2);

  #if VERBOSE
    printf("\nc1:"); mpfq_poly_print_uint64(c1);
    printf("\nc2:"); rbc_poly_print(c2);
    printf("\n");
  #endif

  mu_assert(rbc_poly_equal_to_mpfq(c1, c2) == 1, "Failure: test_poly_mulmod_sparse");
}



MU_TEST(test_poly_div) {
  mpfq_poly_divmod(field1, q1, r1, a1, b1);
  rbc_poly_div(q2, r2, a2, b2);

  #if VERBOSE
    printf("\nq1:"); mpfq_poly_print_uint64(q1);
    printf("\nq2:"); rbc_poly_print(q2);
    printf("\n");
  #endif

  mu_assert(rbc_poly_equal_to_mpfq(q1, q2) == 1 && 
    rbc_poly_equal_to_mpfq(r1, r2) == 1, "Failure: test_poly_div");
}



MU_TEST(test_poly_inv) {
  mpfq_poly gcd;
  mpfq_poly v0;
  mpfq_poly_init(field1, gcd, DEGREE);
  mpfq_poly_init(field1, v0, DEGREE);
  mpfq_poly_xgcd(field1, gcd, c1, v0, a1, modulo1);
  mpfq_poly_clear(field1, gcd);
  mpfq_poly_clear(field1, v0);

  rbc_poly_inv(c2, a2, modulo2);

  mu_assert(rbc_poly_equal_to_mpfq(c1, c2) == 1, "Failure: test_poly_inv");
}



/*
 * Test execution
 */
MU_TEST_SUITE(test_suite) {
  MU_SUITE_CONFIGURE(&test_setup, &test_teardown);
  MU_RUN_TEST(test_poly_add);
  MU_RUN_TEST(test_poly_mul);
  MU_RUN_TEST(test_poly_mulmod_sparse);
  MU_RUN_TEST(test_poly_div);
  MU_RUN_TEST(test_poly_inv);
  MU_SUITE_CLEAR();
}

int main() {
  for(int i = 0 ; i < INSTANCES ; ++i) {
    MU_RUN_SUITE(test_suite);
  }

  MU_REPORT();
  return minunit_fail;
}

