
#ifndef TEST_UTILS_H
#define TEST_UTILS_H


void mpfq_set_random(mpfq_field f, mpfq_elt e) {
  mpfq_set_zero(f, e);

  int bytes = (RBC_FIELD_M % 8 == 0) ? RBC_FIELD_M / 8 : RBC_FIELD_M / 8 + 1;
  unsigned char random[bytes];
  randombytes(random, bytes);

  unsigned char mask = (1 << RBC_FIELD_M % 8) - 1;
  random[bytes - 1] &= mask;
  memcpy((unsigned long*) e, random, bytes);
}



void mpfq_set_random2(mpfq_field f, mpfq_elt_ur e) {
  mpfq_elt_ur_set_zero(f, e);
  int bytes = ((2 * RBC_FIELD_M - 1) % 8 == 0) ? (2 * RBC_FIELD_M - 1) / 8 : (2 * RBC_FIELD_M - 1) / 8 + 1;

  unsigned char random[bytes];
  randombytes(random, bytes);
  unsigned char mask = (1 << ((2 * RBC_FIELD_M - 1) % 8)) - 1;
  random[bytes - 1] &= mask;
  memcpy((unsigned long*) e, random, bytes);
}



void mpfq_elt_print_uint64(mpfq_elt e) {
  int size = RBC_ELT_UINT64;

  printf("[ ");
  for(int i = 0 ; i < size ; ++i) {
    printf("%lu ", e[i]);
  }
  printf("]");
}



void mpfq_elt_ur_print_uint64(mpfq_elt e) {
  int size = RBC_ELT_UR_UINT64;

  printf("[ ");
  for(int i = 0 ; i < size ; ++i) {
    printf("%lu ", e[i]);
  }
  printf("]");
}



int rbc_elt_equal_to_mpfq(mpfq_elt expected, rbc_elt result) {
  int size = RBC_ELT_UINT64;
  if(RBC_FIELD_M == 131 || RBC_FIELD_M == 137 || RBC_FIELD_M == 151 || RBC_FIELD_M == 181) {
    size = RBC_ELT_UINT64 - 1;
  }

  for(int i = 0 ; i < size ; ++i) {
    if(expected[i] != result[i]) {
       return 0;
    }
  }

  return 1;
}



void mpfq_poly_set_zero(mpfq_field f, mpfq_poly p, int degree) {
  mpfq_elt zero;
  mpfq_init(f, zero);
  mpfq_set_zero(f, zero);
  for(int i = 0 ; i <= degree ; i++) {
    mpfq_poly_setcoef(f, p, zero, i);
  }
}



void mpfq_poly_set_random(mpfq_field f, mpfq_poly p, int degree) {
  mpfq_elt random;
  mpfq_init(f, random);
  for(int i = 0 ; i <= degree ; i++) {
    mpfq_set_random(f, random);
    mpfq_poly_setcoef(f, p, random, i);
  }
}



void rbc_poly_set_mpfq(rbc_poly o, mpfq_poly p, int degree) {
  for(int i = 0 ; i <= degree ; i++) {
    rbc_elt_set_from_uint64(o->v[i], p->c[i]);
  }
}



void mpfq_poly_print_uint64(mpfq_poly p) {
  for(int i = 0 ; i < p->size ; i++) {
    printf("\n%i - ", i);
    mpfq_elt_print_uint64(p->c[i]);
  }
  printf("\n");
}



int rbc_poly_equal_to_mpfq(mpfq_poly expected, rbc_poly result) {
  if(expected->size-1 != result->degree) {
    return 0;
  }

  for(int i = 0 ; i <= result->degree ; i++) {
    if(!rbc_elt_equal_to_mpfq(expected->c[i], result->v[i])) {
      return 0;
    }
  }

  return 1;
}

#endif
